#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    intersection.py
#       
#    Copyright 2010 Victor Ramirez <virako.9@gmail.com>
#       
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


from opencv.highgui import *
from opencv.cv import *
from opencv import *
#from math import *
from tablero import Tablero
from check_stone import check_stone
from cte import *
from notation import notation



def contador(lista, num, indice = 0):
    cont = 0
    for l in lista:
        if l[indice] == num:
            cont += 1
    return cont
        


class Intersection:
    """ Contiene todos los datos de las intersecciones de un tablero y las 
       funciones necesarias para encontrarlas. 
       Parámetros necesarios:  
       *image: la imagen del tablero que queremos tratar.
       *template: el template que vamos a pasarle a la imagen. 
       *size: el tamaño del tablero para saber cuantas intersecciones buscar.
    """
       
    def __init__(self, image, template = "images/tplPoint.png", size = SIZE):
        """Inicializamos la imágen, el template y algunas variables. 
        @param image: imagen donde se buscarán las intersecciones del tablero o
         cadena con el directorio donde se encuentra la imagen. 
        @type image: L{IplImage} or C{str}
        @keyword template: imagen de la plantilla que buscaremos en la imagen. 
        @type template:  C{str}
        @keyword size: el tamaño del tablero
        @type size: C{int}
        @return: None
        @rtype: None
        """
        
        if type(image) == str:
            self.img = cvLoadImageM(image, CV_LOAD_IMAGE_GRAYSCALE)
        else:
            self.img =cvCreateImage(cvSize(image.width,image.height), IPL_DEPTH_8U, 1)
            cvCvtColor(image, self.img, CV_BGR2GRAY)

        self.tpl = cvLoadImageM(template, CV_LOAD_IMAGE_GRAYSCALE)
        self.size = size
        self.intersections = []        
        self.tpl_size = [self.tpl.height, self.tpl.width]  # tamaño del template
        self.tablero = Tablero(self.size)
        self.num_jugada = 1
        self.color = BLACK # color de comienzo
        self.player_pass = 0 

    def detect_intersections(self, threshold=0.7):
        """ Función recursiva que detecta las intersecciones del tablero usando 
            un template y un valor umbral, el cual iremos variando para 
            hasta obtener el resultado deseado. 
            @keyword threshold: Umbral para usar la búsqueda del template.
            @type threshold: float.
            """
        i = self.intersections

        #Creamos imágenes auxiliares para utilizar filtros (Canny y Smooth)
        img_aux = cvCreateImage(cvSize(self.img.width, self.img.height), \
         self.img.depth, self.img.nChannels)
        img_aux_2 = cvCreateImage(cvSize(self.img.width, self.img.height), \
         self.img.depth, self.img.nChannels)
        
        tpl_aux = cvCreateImage(cvSize(self.tpl.width, self.tpl.height), \
         self.tpl.depth, self.tpl.nChannels)
        tpl_aux_2 = cvCreateImage(cvSize(self.tpl.width, self.tpl.height), \
         self.tpl.depth, self.tpl.nChannels)
        
        # funcion Canny para detectar bordes de la imagen y Smooth para suavizar
        cvCanny(self.img, img_aux_2, 50,55,3)
        cvSmooth(img_aux_2, img_aux, 3)
        cvCanny(self.tpl, tpl_aux_2, 50,55,3)
        cvSmooth(tpl_aux_2, tpl_aux, 3)
        
        # creamos una nueva imagen para comparar el resultado
        w = self.img.width - self.tpl.width + 1 
        h = self.img.height - self.tpl.height + 1 
        res = cvCreateImage( cvSize(w,h), IPL_DEPTH_32F, 1)
        
        # realizamos una plantilla coincidente
        cvMatchTemplate(img_aux, tpl_aux, res, CV_TM_SQDIFF_NORMED)
        
        # ahora buscamos los objetos coincidentes, variamos threshold al gusto
        for y in range(res.height - 1) :
            for x in range(res.width - 1):
                # obtenemos un elemento
                s = cvGet2D(res, y, x)
                if s[0] < threshold:
                    i.append(((x + x + self.tpl.width)/2, \
                     (y + y + self.tpl.height)/2))
        
        # Eliminamos intersecciones inválidas comparando los puntos encontrados
        repeated = []
        for u in range(len(i)-1):
            for v in range(len(i)-1):
                # Comprobamos que la distancia entre intersecciones sea mayor 
                # que la mitad de un cuadrado, para eliminar las repetidas
                if (u < v and abs(i[u][0] - i[v][0]) < self.tpl.width/2 \
                 and abs(i[u][1] - i[v][1]) < self.tpl.height/2 \
                  and repeated.count(i[v]) != 1):
                    repeated.append(i[v])
                    
        for d in repeated:
            i.remove(d)
        
        if self.size**2 >= len(i) >= (self.size**2)*(2/3.0): 
            print "el numero de intersecciones es ", len(i)
            self.i = i
            return i
        elif len(i) > self.size**2:
            threshold = threshold - 0.05
        elif len(i) < (self.size**2)*(2/3.0):
            threshold = threshold + 0.05
        
        if threshold >= 1.0 or threshold < 0.3:
            print "Inténtalo de nuevo, haz otra foto"
            mal = True
            return -1

        return self.detect_intersections(threshold)
        
        
    def paint_save_intersection(self): 
        """Función que pinta y guarda las intersecciones para chequeos."""
        for p in self.i:
            cvCircle(self.img, cvPoint(p[0],p[1]), 5, cvScalar(255,0,0,0), 1, 8, 0)
        #try: cvSaveImage("images/intersecciones.png", self.img)
        #except: print "Error al guardar la imagen de diferencia. " # para chequeo
        return self.img
    
    
    def media(self): # TODO buscar los errores
        """ Hallamos la media de todas las filas y columnas. """
        
        # medias de las columnas
        x_min = [self.img.width for x in xrange(self.size)]
        x_min.append(0)
        x_media = [0 for x in xrange(self.size + 1)]
        verticales = [[] for x in xrange(self.size)]
        
        # medias de las filas
        y_min = [self.img.height for x in xrange(self.size)]
        y_min.append(0)
        y_media = [0 for x in xrange(self.size + 1)]
        horizontales = [[] for x in xrange(self.size)]
        
        for tam in range(self.size):
            # hallamos la x minima y la y minima
            for i in self.intersections:
                if i[0] < x_min[tam] and i[0] > x_min[tam-1]: # x
                    x_min[tam] = i[0]
                if i[1] < y_min[tam] and i[1] > y_min[tam-1]: # y
                    y_min[tam] = i[1]
            
            # 
            for i in self.intersections:
                if (i[0] >= x_min[tam] and i[0] < x_min[tam] + self.tpl.width/2.0): # x
                    verticales[tam].append(i)
                    if x_media[tam] < i[0]:
                        x_media[tam] = i[0]
            
                if (i[1] >= y_min[tam] and \
                 i[1] < y_min[tam] + self.tpl.height/2.0): # y
                    horizontales[tam].append(i)
                    if y_media[tam] < i[1]:
                        y_media[tam] = i[1]
            x_min[tam] = x_media[tam]
            y_min[tam] = y_media[tam]            
                
        # Guardamos las intersecciones en el tablero 
        for x in range(len(x_media)-1):
            for y in range(len(y_media)-1):
                self.tablero.matriz[x][y][0] = x_media[x]
                self.tablero.matriz[x][y][1] = y_media[y]
                # Pintamos 
                cvCircle(self.img, cvPoint(x_media[x], y_media[y]), 2, \
                 cvScalar(255,0,0,0), 1, 8, 0)
        return self.img
        
    
    def search_intersection(self, points):
        """Busca el punto donde se encuentra la piedra para añadirlo al archivo 
         sgf. 
         @param points: intersecciones donde puede encontrarse la piedra. 
         @type points: C{list(tuples*)}
         @return: Tupla que contiene: 
          Cadena con la intersección donde se encuentra la piedra. 
          Color de la piedra.
          Boleano para saber si hemos terminado la partida. 
         @rtype: C{tuple(str, int, bool}            # Los candidatos los hemos dividido en dos, los que estaban en 
            # intersecciones libres, y los que estaban en ocupadas.
         """
        
        candidates = []
        self.tablero.get_square() # Obtenemos el tamaño de los cuadrados 
        sgf_stone = ""
        ocupadas = []
        libres = []
        
        # De la lista de circunferencias que pasamos por parámetro a la función
        # buscamos los candidatos que se encuentren en intersecciones correctas
        if len(points) != 0:
            for point in points:
                candidate = [-1, -1] # inicializamos a -1 para comprobar fallos
                for diagonal in range(19): 
                    point_diagonal = self.tablero.matriz[diagonal][diagonal]
                    if abs(point_diagonal[0]-point[0]) < self.tablero.square[0]/2:
                        candidate[0] = diagonal
                    if abs(point_diagonal[1]-point[1]) < self.tablero.square[1]/2:
                        candidate[1] = diagonal
                    
                if (candidate[0] != -1 and candidate[1] != -1):
                    candidates.append((candidate[0], candidate[1]))
                else:
                    print "Punto", point, "eliminado"
            print "Hay ", len(candidates), "piedras candidatas"
            
            # Una vez obtenidos los candidatos, los recorremos para comprobar 
            # que no son piedras que ya estaban puestas, o si son piedras que ya
            # estaban puestas, comprobar si han sido capturadas
            # Los candidatos los dividimos en dos, los que están en 
            # intersecciones libres, y los que están en intersecciones ocupadas.
            for cand in candidates:
                ocupado = (self.tablero.matriz[cand[0]][cand[1]][2].values()[-1] != SPACE)
                if ocupado:
                    ocupadas.append(cand)
                    print "OCUPADA", cand[0]+1, cand[1]+1
                else:
                    libres.append(cand)
                    print "LIBRE", cand[0]+1, cand[1]+1
            
            # Si solo nos hemos quedado con una piedra libre, la añadimos 
            if len(libres) == 1:
                self.tablero.matriz[libres[0][0]][libres[0][1]][2].update({self.num_jugada:self.color})
                sgf_stone = notation((libres[0][0], libres[0][1]))
            # en el caso de habernos quedado sin ninguna, pasamos (aunque puede
            # ser que en vez de pasar no se haya detectado la piedra)
            elif len(libres) == 0:
                self.player_pass += 1
                print "El jugador ha pasado o la piedra no se ha encontrado. \
                 No existian piedras libres"
            # en el caso de obtener más de una piedra en huecos libres pueden 
            # suceder dos cosas: 
            # * que se nos haya olvidado pulsar el reloj, y haya dos piedras
            # * que hayamos encontrado piedras erróneas
            else:  
                if self.player_pass == 1: # TODO si ocurre esto, modificar el sgf
                    print "La piedra anterior no se había pasado"
                else:
                    print "Demasiados huecos libres: ", len(libres)
                    exit(0)
                    
            # Los candidatos que se encuentran en posiciones ocupadas 
            # tenemos que recorrerlos todos y comprobar que si las piedras
            # han sido capturarlas para eliminarlas del tablero o no
            for o in ocupadas:
                if check_stone(self.tablero, o, self.color):
                    print "Piedra capturada", o
                    self.tablero.matriz[cand[0]][cand[1]][2].update({self.num_jugada:SPACE})   
                else:
                    print "Hemos comprobado que era Falsa la piedra que podia ser capturada ^^ ", o
            
        else: # No existian cincunferencias donde buscar
            self.player_pass += 1
            print "El jugador ha pasado o la piedra no se ha encontrado.\
             No existian cincunferencias donde buscar."
        
        # Comprobamos cuantas veces ha pasado el jugador para saber si ha 
        # terminado la partida. 
        if self.player_pass == 2:
            r_tupla = (sgf_stone, self.color, True)
        else:
            r_tupla = (sgf_stone, self.color, False)
            
        self.num_jugada += 1
        # intercambia entre negra y blanca
        if self.color == BLACK:     self.color = WHITE
        elif self.color == WHITE:   self.color = BLACK
        
        return r_tupla

                

