#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    luminosity.py
#       
#    Copyright 2010 Victor Ramirez <virako.9@gmail.com>
#       
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.



from opencv.highgui import *
from opencv.cv import *
from opencv import *



def luminosity(image, tablero, size, lum = 150):
    """Funcion que busca los puntos luminosos de la imagen y los devuelve. 
       Esta función ya no se usa por que existian varios factores en contra.
       Factor tiempo: tardaba mucho en recorrer muchos puntos (aunque con alg. 
       de busqueda se podía bajar ese tiempo).
       Factor error: como cogía todos los puntos luminosos, cuando aparecía una 
       pequeña sombra se producían muchos errores y a la vez cientos de puntos 
       mas que recorrer"""
    
    puntos = []
    # buscamos los 4 vértices del tablero para buscar solo dentro de ellos
    x1 = tablero.matriz[0][0][0] - tablero.square[0]
    y1 = tablero.matriz[0][1][1] - tablero.square[1]
    x2 = tablero.matriz[size-1][size-1][0] + tablero.square[0]
    y2 = tablero.matriz[size-1][size-1][1] + tablero.square[1]
      
    for x in range(x1, x2):
        for y in range(y1, y2):
            if (image[x][y][0] + image[x][y][1] + image[x][y][2]) > lum:
                puntos.append((x,y))
    return puntos
    
def search_circle(image, tablero, size, dp = 1.6):
    """Función que trata la imagen luminosa con los filtros Canny y Smooth y 
       busca cincunferencias. Devuelve los centros de las cincunferencias 
       encontradas """ 
    
    stones = []

    gray = cvCreateImage(cvSize(image.width, image.height), IPL_DEPTH_8U, 1)
    cvCvtColor(image, gray, CV_BGR2GRAY)
    
    # creamos dos imagenes auxiliares para aplicar los filtros
    gray_aux = cvCreateImage(cvSize(gray.width, gray.height), gray.depth, \
     gray.nChannels)
    gray_aux_2 = cvCreateImage(cvSize(gray.width, gray.height), gray.depth, \
     gray.nChannels)
    
    # buscamos los 4 vértices del tablero para buscar solo dentro de ellos # TODO
    #x1 = tablero.matriz[0][0][0] - tablero.square[0]
    #y1 = tablero.matriz[0][1][1] - tablero.square[1]
    #x2 = tablero.matriz[size-1][size-1][0] + tablero.square[0]
    #y2 = tablero.matriz[size-1][size-1][1] + tablero.square[1]
    
    cvCanny(gray, gray_aux_2, 50,55,3)
    cvSmooth(gray_aux_2, gray_aux, CV_GAUSSIAN, 5, 5)
    
    # creo una matriz de para guardar los circulos encontrados
    storage = cvCreateMat(1, gray_aux.height*gray_aux.width, cv.CV_32FC3)

    # Buscamos circunferencias en la imagen con la transformada de Hough, 
    # el tamaño de las circunferencias estarán entre los radios dados
    tablero.get_square()
    circles = cvHoughCircles( gray_aux, storage, CV_HOUGH_GRADIENT, dp, \
     tablero.square[0]/2, 50, 55, tablero.square[0]/4,  tablero.square[0] ) 
    
    try:
        for n in range(0, storage.cols):
            p = cvGet1D(storage, n)
            pt = (cvRound(p[0]), cvRound(p[1]))
            stones.append(pt)
            cvCircle(gray, pt, cvRound(p[2]), cv.CV_RGB(255, 0, 0), 2) # pinta el circulo encontrado
        print "Hay ", len(stones), " circulos en la imagen"
    except:
        print "No hemos podido recorrer el storage de cvHoughCircles.\
         No se han circunferencias"
    
    #try: cvSaveImage("images/circunferencias.png", gray)
    #except: print "Error al guardar la imagen con cincunferencias. " # para chequeo
    
    #if storage.cols == 0 and dp <= 2 and len(stones) == 0:
    #    dp += 0.2
    #    print "variando profundidad para buscar mejor:", dp
        
    #    return search_circle(image, tablero, size, dp = dp+0.2) # TODO problema al hacer la recursividad y al variar el dp
    #    stones.append((100,100))
        
    return stones

